@@ -21,8 +21,9 @@
 #include <stdint.h>
 #include <stdio.h>
 
+#include <cmath>  // std::abs
 #include <memory>
-#include <numeric>  // iota
+#include <numeric>  // std::iota, std::inner_product
 
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "hwy/examples/benchmark.cc"
@@ -143,7 +144,10 @@ class BenchmarkDot : public TwoArray {
     sum0 = Add(sum0, sum1);
     sum2 = Add(sum2, sum3);
     sum0 = Add(sum0, sum2);
+    // Remember to store the result in `dot_` for verification; see `Verify`.
     dot_ = GetLane(SumOfLanes(d, sum0));
+    // Return the result so that the benchmarking framework can ensure that the
+    // computation is not elided by the compiler.
     return static_cast<FuncOutput>(dot_);
   }
   void Verify(size_t num_items) {
