@@ -24,14 +24,15 @@
 #include <stdlib.h>
 #include <time.h>    // clock_gettime
 
-#include <algorithm>  // sort
+#include <algorithm>  // std::sort, std::find_if
 #include <array>
 #include <atomic>
 #include <chrono>  //NOLINT
 #include <limits>
-#include <numeric>  // iota
+#include <numeric>  // std::iota
 #include <random>
 #include <string>
+#include <utility>  // std::pair
 #include <vector>
 
 #if defined(_WIN32) || defined(_WIN64)
@@ -150,7 +151,7 @@ inline Ticks Start() {
       // "cc" = flags modified by SHL.
       : "rdx", "memory", "cc");
 #elif HWY_ARCH_RVV
-  asm volatile("rdcycle %0" : "=r"(t));
+  asm volatile("rdtime %0" : "=r"(t));
 #elif defined(_WIN32) || defined(_WIN64)
   LARGE_INTEGER counter;
   (void)QueryPerformanceCounter(&counter);
@@ -325,24 +326,6 @@ T MedianAbsoluteDeviation(const T* values, const size_t num_values,
 namespace platform {
 namespace {
 
-// Prevents the compiler from eliding the computations that led to "output".
-template <class T>
-inline void PreventElision(T&& output) {
-#if HWY_COMPILER_MSVC == 0
-  // Works by indicating to the compiler that "output" is being read and
-  // modified. The +r constraint avoids unnecessary writes to memory, but only
-  // works for built-in types (typically FuncOutput).
-  asm volatile("" : "+r"(output) : : "memory");
-#else
-  // MSVC does not support inline assembly anymore (and never supported GCC's
-  // RTL constraints). Self-assignment with #pragma optimize("off") might be
-  // expected to prevent elision, but it does not with MSVC 2015. Type-punning
-  // with volatile pointers generates inefficient code on MSVC 2017.
-  static std::atomic<T> dummy(T{});
-  dummy.store(output, std::memory_order_relaxed);
-#endif
-}
-
 // Measures the actual current frequency of Ticks. We cannot rely on the nominal
 // frequency encoded in x86 BrandString because it is misleading on M1 Rosetta,
 // and not reported by AMD. CPUID 0x15 is also not yet widely supported. Also
@@ -576,7 +559,7 @@ size_t NumSkip(const Func func, const uint8_t* arg, const InputVec& unique,
     double rel_mad;
     const timer::Ticks total = SampleUntilStable(
         p.target_rel_mad, &rel_mad, p,
-        [func, arg, input]() { platform::PreventElision(func(arg, input)); });
+        [func, arg, input]() { PreventElision(func(arg, input)); });
     min_duration = HWY_MIN(min_duration, total - timer_resolution);
   }
 
@@ -664,7 +647,7 @@ timer::Ticks TotalDuration(const Func func, const uint8_t* arg,
   const timer::Ticks duration =
       SampleUntilStable(p.target_rel_mad, &rel_mad, p, [func, arg, inputs]() {
         for (const FuncInput input : *inputs) {
-          platform::PreventElision(func(arg, input));
+          PreventElision(func(arg, input));
         }
       });
   *max_rel_mad = HWY_MAX(*max_rel_mad, rel_mad);
@@ -684,7 +667,7 @@ timer::Ticks Overhead(const uint8_t* arg, const InputVec* inputs,
   // Zero tolerance because repeatability is crucial and EmptyFunc is fast.
   return SampleUntilStable(0.0, &rel_mad, p, [arg, inputs]() {
     for (const FuncInput input : *inputs) {
-      platform::PreventElision(EmptyFunc(arg, input));
+      PreventElision(EmptyFunc(arg, input));
     }
   });
 }
