@@ -13,7 +13,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <inttypes.h>
 #include <stddef.h>
 #include <stdint.h>
 
@@ -30,9 +29,9 @@ namespace HWY_NAMESPACE {
 struct TestPlusMinus {
   template <class T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
-    const auto v2 = Iota(d, T(2));
-    const auto v3 = Iota(d, T(3));
-    const auto v4 = Iota(d, T(4));
+    const auto v2 = Iota(d, T{2});
+    const auto v3 = Iota(d, T{3});
+    const auto v4 = Iota(d, T{4});
 
     const size_t N = Lanes(d);
     auto lanes = AllocateAligned<T>(N);
@@ -40,7 +39,7 @@ struct TestPlusMinus {
       lanes[i] = static_cast<T>((2 + i) + (3 + i));
     }
     HWY_ASSERT_VEC_EQ(d, lanes.get(), Add(v2, v3));
-    HWY_ASSERT_VEC_EQ(d, Set(d, 2), Sub(v4, v2));
+    HWY_ASSERT_VEC_EQ(d, Set(d, T{2}), Sub(v4, v2));
 
     for (size_t i = 0; i < N; ++i) {
       lanes[i] = static_cast<T>((2 + i) + (4 + i));
@@ -54,15 +53,32 @@ struct TestPlusMinus {
   }
 };
 
+struct TestPlusMinusOverflow {
+  template <class T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const auto v1 = Iota(d, T(1));
+    const auto vMax = Iota(d, LimitsMax<T>());
+    const auto vMin = Iota(d, LimitsMin<T>());
+
+    // Check that no UB triggered.
+    // "assert" here is formal - to avoid compiler dropping calculations
+    HWY_ASSERT_VEC_EQ(d, Add(v1, vMax), Add(vMax, v1));
+    HWY_ASSERT_VEC_EQ(d, Add(vMax, vMax), Add(vMax, vMax));
+    HWY_ASSERT_VEC_EQ(d, Sub(vMin, v1), Sub(vMin, v1));
+    HWY_ASSERT_VEC_EQ(d, Sub(vMin, vMax), Sub(vMin, vMax));
+  }
+};
+
 HWY_NOINLINE void TestAllPlusMinus() {
   ForAllTypes(ForPartialVectors<TestPlusMinus>());
+  ForIntegerTypes(ForPartialVectors<TestPlusMinusOverflow>());
 }
 
 struct TestUnsignedSaturatingArithmetic {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     const auto v0 = Zero(d);
-    const auto vi = Iota(d, 1);
+    const auto vi = Iota(d, T{1});
     const auto vm = Set(d, LimitsMax<T>());
 
     HWY_ASSERT_VEC_EQ(d, Add(v0, v0), SaturatedAdd(v0, v0));
@@ -85,7 +101,7 @@ struct TestSignedSaturatingArithmetic {
     const auto v0 = Zero(d);
     const auto vpm = Set(d, LimitsMax<T>());
     // Ensure all lanes are positive, even if Iota wraps around
-    const auto vi = Or(And(Iota(d, 0), vpm), Set(d, 1));
+    const auto vi = Or(And(Iota(d, 0), vpm), Set(d, T{1}));
     const auto vn = Sub(v0, vi);
     const auto vnm = Set(d, LimitsMin<T>());
     HWY_ASSERT_MASK_EQ(d, MaskTrue(d), Gt(vi, v0));
@@ -105,6 +121,26 @@ struct TestSignedSaturatingArithmetic {
   }
 };
 
+struct TestSaturatingArithmeticOverflow {
+  template <class T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const auto v1 = Iota(d, T(1));
+    const auto vMax = Iota(d, LimitsMax<T>());
+    const auto vMin = Iota(d, LimitsMin<T>());
+
+    // Check that no UB triggered.
+    // "assert" here is formal - to avoid compiler dropping calculations
+    HWY_ASSERT_VEC_EQ(d, SaturatedAdd(v1, vMax), SaturatedAdd(vMax, v1));
+    HWY_ASSERT_VEC_EQ(d, SaturatedAdd(vMax, vMax), SaturatedAdd(vMax, vMax));
+    HWY_ASSERT_VEC_EQ(d, SaturatedAdd(vMin, vMax), SaturatedAdd(vMin, vMax));
+    HWY_ASSERT_VEC_EQ(d, SaturatedAdd(vMin, vMin), SaturatedAdd(vMin, vMin));
+    HWY_ASSERT_VEC_EQ(d, SaturatedSub(vMin, v1), SaturatedSub(vMin, v1));
+    HWY_ASSERT_VEC_EQ(d, SaturatedSub(vMin, vMax), SaturatedSub(vMin, vMax));
+    HWY_ASSERT_VEC_EQ(d, SaturatedSub(vMax, vMin), SaturatedSub(vMax, vMin));
+    HWY_ASSERT_VEC_EQ(d, SaturatedSub(vMin, vMin), SaturatedSub(vMin, vMin));
+  }
+};
+
 HWY_NOINLINE void TestAllSaturatingArithmetic() {
   const ForPartialVectors<TestUnsignedSaturatingArithmetic> test_unsigned;
   test_unsigned(uint8_t());
@@ -113,14 +149,20 @@ HWY_NOINLINE void TestAllSaturatingArithmetic() {
   const ForPartialVectors<TestSignedSaturatingArithmetic> test_signed;
   test_signed(int8_t());
   test_signed(int16_t());
+
+  const ForPartialVectors<TestSaturatingArithmeticOverflow> test_overflow;
+  test_overflow(int8_t());
+  test_overflow(uint8_t());
+  test_overflow(int16_t());
+  test_overflow(uint16_t());
 }
 
 struct TestAverage {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     const auto v0 = Zero(d);
-    const auto v1 = Set(d, T(1));
-    const auto v2 = Set(d, T(2));
+    const auto v1 = Set(d, T{1});
+    const auto v2 = Set(d, T{2});
 
     HWY_ASSERT_VEC_EQ(d, v0, AverageRound(v0, v0));
     HWY_ASSERT_VEC_EQ(d, v1, AverageRound(v0, v1));
@@ -140,8 +182,8 @@ struct TestAbs {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     const auto v0 = Zero(d);
-    const auto vp1 = Set(d, T(1));
-    const auto vn1 = Set(d, T(-1));
+    const auto vp1 = Set(d, T{1});
+    const auto vn1 = Set(d, T{-1});
     const auto vpm = Set(d, LimitsMax<T>());
     const auto vnm = Set(d, LimitsMin<T>());
 
@@ -157,10 +199,10 @@ struct TestFloatAbs {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     const auto v0 = Zero(d);
-    const auto vp1 = Set(d, T(1));
-    const auto vn1 = Set(d, T(-1));
-    const auto vp2 = Set(d, T(0.01));
-    const auto vn2 = Set(d, T(-0.01));
+    const auto vp1 = Set(d, T{1});
+    const auto vn1 = Set(d, T{-1});
+    const auto vp2 = Set(d, static_cast<T>(0.01));
+    const auto vn2 = Set(d, static_cast<T>(-0.01));
 
     HWY_ASSERT_VEC_EQ(d, v0, Abs(v0));
     HWY_ASSERT_VEC_EQ(d, vp1, Abs(vp1));
@@ -179,17 +221,28 @@ struct TestNeg {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     const auto v0 = Zero(d);
-    const auto vn = Set(d, T(-3));
-    const auto vp = Set(d, T(3));
+    const auto vn = Set(d, T{-3});
+    const auto vp = Set(d, T{3});
     HWY_ASSERT_VEC_EQ(d, v0, Neg(v0));
     HWY_ASSERT_VEC_EQ(d, vp, Neg(vn));
     HWY_ASSERT_VEC_EQ(d, vn, Neg(vp));
   }
 };
 
+struct TestNegOverflow {
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const auto vn = Set(d, LimitsMin<T>());
+    const auto vp = Set(d, LimitsMax<T>());
+    HWY_ASSERT_VEC_EQ(d, Neg(vn), Neg(vn));
+    HWY_ASSERT_VEC_EQ(d, Neg(vp), Neg(vp));
+  }
+};
+
 HWY_NOINLINE void TestAllNeg() {
   ForSignedTypes(ForPartialVectors<TestNeg>());
   ForFloatTypes(ForPartialVectors<TestNeg>());
+  ForSignedTypes(ForPartialVectors<TestNegOverflow>());
 }
 
 struct TestUnsignedMinMax {
@@ -198,8 +251,8 @@ struct TestUnsignedMinMax {
     const auto v0 = Zero(d);
     // Leave headroom such that v1 < v2 even after wraparound.
     const auto mod = And(Iota(d, 0), Set(d, LimitsMax<T>() >> 1));
-    const auto v1 = Add(mod, Set(d, 1));
-    const auto v2 = Add(mod, Set(d, 2));
+    const auto v1 = Add(mod, Set(d, T{1}));
+    const auto v2 = Add(mod, Set(d, T{2}));
     HWY_ASSERT_VEC_EQ(d, v1, Min(v1, v2));
     HWY_ASSERT_VEC_EQ(d, v2, Max(v1, v2));
     HWY_ASSERT_VEC_EQ(d, v0, Min(v1, v0));
@@ -220,9 +273,10 @@ struct TestSignedMinMax {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     // Leave headroom such that v1 < v2 even after wraparound.
-    const auto mod = And(Iota(d, 0), Set(d, LimitsMax<T>() >> 1));
-    const auto v1 = Add(mod, Set(d, 1));
-    const auto v2 = Add(mod, Set(d, 2));
+    const auto mod =
+        And(Iota(d, 0), Set(d, static_cast<T>(LimitsMax<T>() >> 1)));
+    const auto v1 = Add(mod, Set(d, T{1}));
+    const auto v2 = Add(mod, Set(d, T{2}));
     const auto v_neg = Sub(Zero(d), v1);
     HWY_ASSERT_VEC_EQ(d, v1, Min(v1, v2));
     HWY_ASSERT_VEC_EQ(d, v2, Max(v1, v2));
@@ -257,8 +311,8 @@ struct TestFloatMinMax {
     HWY_ASSERT_VEC_EQ(d, v1, Max(v1, v_neg));
 
     const auto v0 = Zero(d);
-    const auto vmin = Set(d, T(-1E30));
-    const auto vmax = Set(d, T(1E30));
+    const auto vmin = Set(d, static_cast<T>(-1E30));
+    const auto vmax = Set(d, static_cast<T>(1E30));
     HWY_ASSERT_VEC_EQ(d, vmin, Min(v0, vmin));
     HWY_ASSERT_VEC_EQ(d, vmin, Min(vmin, v0));
     HWY_ASSERT_VEC_EQ(d, v0, Max(v0, vmin));
@@ -424,6 +478,60 @@ HWY_NOINLINE void TestAllMinMax128Upper() {
   ForGEVectors<128, TestMinMax128Upper>()(uint64_t());
 }
 
+struct TestIntegerAbsDiff {
+  template<typename T, HWY_IF_T_SIZE_ONE_OF(T, (1 << 1) | (1 << 2) | (1 << 4))>
+  static inline T ScalarAbsDiff(T a, T b) {
+    using TW = MakeSigned<MakeWide<T>>;
+    const TW diff = static_cast<TW>(a) - static_cast<TW>(b);
+    return static_cast<T>((diff >= 0) ? diff : -diff);
+  }
+  template<typename T, HWY_IF_T_SIZE(T, 8)>
+  static inline T ScalarAbsDiff(T a, T b) {
+    if (a >= b) {
+      return a - b;
+    } else {
+      return b - a;
+    }
+  }
+
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const size_t N = Lanes(d);
+    auto in_lanes_a = AllocateAligned<T>(N);
+    auto in_lanes_b = AllocateAligned<T>(N);
+    auto out_lanes = AllocateAligned<T>(N);
+    constexpr size_t shift_amt_mask = sizeof(T) * 8 - 1;
+    for (size_t i = 0; i < N; ++i) {
+      // Need to mask out shift_amt as i can be greater than or equal to
+      // the number of bits in T if T is int8_t, uint8_t, int16_t, or uint16_t.
+      const auto shift_amt = i & shift_amt_mask;
+      in_lanes_a[i] =
+        static_cast<T>((static_cast<uint64_t>(i) ^ 1u) << shift_amt);
+      in_lanes_b[i] =
+        static_cast<T>(static_cast<uint64_t>(i) << shift_amt);
+      out_lanes[i] = ScalarAbsDiff(in_lanes_a[i], in_lanes_b[i]);
+    }
+    const auto a = Load(d, in_lanes_a.get());
+    const auto b = Load(d, in_lanes_b.get());
+    const auto expected = Load(d, out_lanes.get());
+    HWY_ASSERT_VEC_EQ(d, expected, AbsDiff(a, b));
+    HWY_ASSERT_VEC_EQ(d, expected, AbsDiff(b, a));
+  }
+};
+
+HWY_NOINLINE void TestAllIntegerAbsDiff() {
+  ForPartialVectors<TestIntegerAbsDiff>()(int8_t());
+  ForPartialVectors<TestIntegerAbsDiff>()(uint8_t());
+  ForPartialVectors<TestIntegerAbsDiff>()(int16_t());
+  ForPartialVectors<TestIntegerAbsDiff>()(uint16_t());
+  ForPartialVectors<TestIntegerAbsDiff>()(int32_t());
+  ForPartialVectors<TestIntegerAbsDiff>()(uint32_t());
+#if HWY_HAVE_INTEGER64
+  ForPartialVectors<TestIntegerAbsDiff>()(int64_t());
+  ForPartialVectors<TestIntegerAbsDiff>()(uint64_t());
+#endif
+}
+
 // NOLINTNEXTLINE(google-readability-namespace-comments)
 }  // namespace HWY_NAMESPACE
 }  // namespace hwy
@@ -441,6 +549,7 @@ HWY_EXPORT_AND_TEST_P(HwyArithmeticTest, TestAllNeg);
 HWY_EXPORT_AND_TEST_P(HwyArithmeticTest, TestAllMinMax);
 HWY_EXPORT_AND_TEST_P(HwyArithmeticTest, TestAllMinMax128);
 HWY_EXPORT_AND_TEST_P(HwyArithmeticTest, TestAllMinMax128Upper);
+HWY_EXPORT_AND_TEST_P(HwyArithmeticTest, TestAllIntegerAbsDiff);
 }  // namespace hwy
 
 #endif
