@@ -16,6 +16,9 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <string.h>
+
+#include <cmath>  // std::isfinite
+
 #include "hwy/base.h"
 
 #undef HWY_TARGET_INCLUDE
@@ -28,6 +31,42 @@ HWY_BEFORE_NAMESPACE();
 namespace hwy {
 namespace HWY_NAMESPACE {
 
+template <typename T, size_t N, int kPow2>
+size_t DeduceN(Simd<T, N, kPow2>) {
+  return N;
+}
+
+template <typename ToT>
+struct TestRebind {
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const Rebind<ToT, D> dto;
+    const size_t N = Lanes(d);
+    HWY_ASSERT(N <= MaxLanes(d));
+    const size_t NTo = Lanes(dto);
+    if (NTo != N) {
+      HWY_ABORT("u%zu -> u%zu: lanes %zu %zu pow2 %d %d cap %zu %zu\n",
+                8 * sizeof(T), 8 * sizeof(ToT), N, NTo, d.Pow2(), dto.Pow2(),
+                DeduceN(d), DeduceN(dto));
+    }
+  }
+};
+
+// Lane count remains the same when we rebind to smaller/equal/larger types.
+HWY_NOINLINE void TestAllRebind() {
+#if HWY_HAVE_INTEGER64
+  ForShrinkableVectors<TestRebind<uint8_t>, 3>()(uint64_t());
+#endif  // HWY_HAVE_INTEGER64
+  ForShrinkableVectors<TestRebind<uint8_t>, 2>()(uint32_t());
+  ForShrinkableVectors<TestRebind<uint8_t>, 1>()(uint16_t());
+  ForPartialVectors<TestRebind<uint8_t>>()(uint8_t());
+  ForExtendableVectors<TestRebind<uint16_t>, 1>()(uint8_t());
+  ForExtendableVectors<TestRebind<uint32_t>, 2>()(uint8_t());
+#if HWY_HAVE_INTEGER64
+  ForExtendableVectors<TestRebind<uint64_t>, 3>()(uint8_t());
+#endif  // HWY_HAVE_INTEGER64
+}
+
 // Cast and ensure bytes are the same. Called directly from TestAllBitCast or
 // via TestBitCastFrom.
 template <typename ToT>
@@ -155,7 +194,7 @@ struct TestPromoteTo {
     for (size_t rep = 0; rep < AdjustedReps(200); ++rep) {
       for (size_t i = 0; i < N; ++i) {
         const uint64_t bits = rng();
-        memcpy(&from[i], &bits, sizeof(T));
+        CopyBytes<sizeof(T)>(&bits, &from[i]);  // not same size
         expected[i] = from[i];
       }
 
@@ -235,13 +274,19 @@ AlignedFreeUniquePtr<float[]> F16TestCases(D d, size_t& padded) {
       -2.00390625f, -3.99609375f,
       // No infinity/NaN - implementation-defined due to ARM.
   };
-  const size_t kNumTestCases = sizeof(test_cases) / sizeof(test_cases[0]);
+  constexpr size_t kNumTestCases = sizeof(test_cases) / sizeof(test_cases[0]);
   const size_t N = Lanes(d);
+  HWY_ASSERT(N != 0);
   padded = RoundUpTo(kNumTestCases, N);  // allow loading whole vectors
   auto in = AllocateAligned<float>(padded);
   auto expected = AllocateAligned<float>(padded);
-  std::copy(test_cases, test_cases + kNumTestCases, in.get());
-  std::fill(in.get() + kNumTestCases, in.get() + padded, 0.0f);
+  size_t i = 0;
+  for (; i < kNumTestCases; ++i) {
+    in[i] = test_cases[i];
+  }
+  for (; i < padded; ++i) {
+    in[i] = 0.0f;
+  }
   return in;
 }
 
@@ -250,10 +295,11 @@ struct TestF16 {
   HWY_NOINLINE void operator()(TF32 /*t*/, DF32 d32) {
 #if HWY_HAVE_FLOAT16
     size_t padded;
+    const size_t N = Lanes(d32);  // same count for f16
+    HWY_ASSERT(N != 0);
     auto in = F16TestCases(d32, padded);
     using TF16 = float16_t;
     const Rebind<TF16, DF32> d16;
-    const size_t N = Lanes(d32);  // same count for f16
     auto temp16 = AllocateAligned<TF16>(N);
 
     for (size_t i = 0; i < padded; i += N) {
@@ -289,13 +335,19 @@ AlignedFreeUniquePtr<float[]> BF16TestCases(D d, size_t& padded) {
       // negative +/- delta
       -2.015625f, -3.984375f,
   };
-  const size_t kNumTestCases = sizeof(test_cases) / sizeof(test_cases[0]);
+  constexpr size_t kNumTestCases = sizeof(test_cases) / sizeof(test_cases[0]);
   const size_t N = Lanes(d);
+  HWY_ASSERT(N != 0);
   padded = RoundUpTo(kNumTestCases, N);  // allow loading whole vectors
   auto in = AllocateAligned<float>(padded);
   auto expected = AllocateAligned<float>(padded);
-  std::copy(test_cases, test_cases + kNumTestCases, in.get());
-  std::fill(in.get() + kNumTestCases, in.get() + padded, 0.0f);
+  size_t i = 0;
+  for (; i < kNumTestCases; ++i) {
+    in[i] = test_cases[i];
+  }
+  for (; i < padded; ++i) {
+    in[i] = 0.0f;
+  }
   return in;
 }
 
@@ -313,7 +365,8 @@ struct TestBF16 {
 #endif
     const Half<decltype(dbf16)> dbf16_half;
     const size_t N = Lanes(d32);
-    HWY_ASSERT(Lanes(dbf16_half) <= N);
+
+    HWY_ASSERT(Lanes(dbf16_half) == N);
     auto temp16 = AllocateAligned<TBF16>(N);
 
     for (size_t i = 0; i < padded; i += N) {
@@ -349,7 +402,7 @@ HWY_NOINLINE void TestAllConvertU8() {
 template <typename From, typename To, class D>
 constexpr bool IsSupportedTruncation() {
   return (sizeof(To) < sizeof(From)) &&
-         (Pow2(Rebind<To, D>()) + 3 >= static_cast<int>(CeilLog2(sizeof(To))));
+         (Rebind<To, D>().Pow2() + 3 >= static_cast<int>(CeilLog2(sizeof(To))));
 }
 
 struct TestTruncateTo {
@@ -387,10 +440,13 @@ HWY_NOINLINE void TestAllTruncate() {
 struct TestIntFromFloatHuge {
   template <typename TF, class DF>
   HWY_NOINLINE void operator()(TF /*unused*/, const DF df) {
-    // Still does not work, although ARMv7 manual says that float->int
-    // saturates, i.e. chooses the nearest representable value. Also causes
-    // out-of-memory for MSVC.
-#if HWY_TARGET != HWY_NEON && !HWY_COMPILER_MSVC
+    // The ARMv7 manual says that float->int saturates, i.e. chooses the
+    // nearest representable value. This works correctly on armhf with GCC, but
+    // not with clang. For reasons unknown, MSVC also runs into an out-of-memory
+    // error here.
+#if HWY_COMPILER_CLANG || HWY_COMPILER_MSVC
+    (void)df;
+#else
     using TI = MakeSigned<TF>;
     const Rebind<TI, DF> di;
 
@@ -406,8 +462,6 @@ struct TestIntFromFloatHuge {
     // Huge negative
     Store(Set(di, LimitsMin<TI>()), di, expected.get());
     HWY_ASSERT_VEC_EQ(di, expected.get(), ConvertTo(di, Set(df, TF(-1E20))));
-#else
-    (void)df;
 #endif
   }
 };
@@ -451,7 +505,7 @@ class TestIntFromFloat {
       for (size_t i = 0; i < N; ++i) {
         do {
           const uint64_t bits = rng();
-          memcpy(&from[i], &bits, sizeof(TF));
+          CopyBytes<sizeof(TF)>(&bits, &from[i]);  // not same size
         } while (!std::isfinite(from[i]));
         if (from[i] >= max) {
           expected[i] = LimitsMax<TI>();
@@ -532,6 +586,34 @@ HWY_NOINLINE void TestAllFloatFromInt() {
   ForFloatTypes(ForPartialVectors<TestFloatFromInt>());
 }
 
+struct TestFloatFromUint {
+  template <typename TF, class DF>
+  HWY_NOINLINE void operator()(TF /*unused*/, const DF df) {
+    using TU = MakeUnsigned<TF>;
+    const RebindToUnsigned<DF> du;
+
+    // Integer positive
+    HWY_ASSERT_VEC_EQ(df, Iota(df, TF(4.0)), ConvertTo(df, Iota(du, TU(4))));
+    HWY_ASSERT_VEC_EQ(df, Iota(df, TF(65535.0)),
+                      ConvertTo(df, Iota(du, 65535)));  // 2^16-1
+    if (sizeof(TF) > 4) {
+      HWY_ASSERT_VEC_EQ(df, Iota(df, TF(4294967295.0)),
+                        ConvertTo(df, Iota(du, 4294967295ULL)));  // 2^32-1
+    }
+
+    // Max positive
+    HWY_ASSERT_VEC_EQ(df, Set(df, TF(LimitsMax<TU>())),
+                      ConvertTo(df, Set(du, LimitsMax<TU>())));
+
+    // Zero
+    HWY_ASSERT_VEC_EQ(df, Zero(df), ConvertTo(df, Zero(du)));
+  }
+};
+
+HWY_NOINLINE void TestAllFloatFromUint() {
+  ForFloatTypes(ForPartialVectors<TestFloatFromUint>());
+}
+
 struct TestI32F64 {
   template <typename TF, class DF>
   HWY_NOINLINE void operator()(TF /*unused*/, const DF df) {
@@ -583,6 +665,7 @@ HWY_AFTER_NAMESPACE();
 
 namespace hwy {
 HWY_BEFORE_TEST(HwyConvertTest);
+HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllRebind);
 HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllBitCast);
 HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllPromoteTo);
 HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllF16);
@@ -591,6 +674,7 @@ HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllConvertU8);
 HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllTruncate);
 HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllIntFromFloat);
 HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllFloatFromInt);
+HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllFloatFromUint);
 HWY_EXPORT_AND_TEST_P(HwyConvertTest, TestAllI32F64);
 }  // namespace hwy
 
